@@ -1,7 +1,6 @@
-# Apart from the three lines below, it is actually a -*- perl -*- code.
-eval '(exit $?0)' && eval 'exec perl -wS "$0" ${1+"$@"}'
-  & eval 'exec perl -wS "$0" $argv:q'
-    if 0;
+#!/usr/bin/perl
+
+# $Id: msg2smtp.pl,v 1.8 2007/07/29 16:30:25 polak Exp $
 
 my $usage = qq!
 $0 takes a mail message on STDIN and relays it to an SMTP server.
@@ -33,7 +32,7 @@ $0 -h HOST [options]
 # 0. GPL License
 #
 #  This file is part of GNU Anubis.
-#  Copyright (C) 2001-2020 The Anubis Team.
+#  Copyright (C) 2001, 2002, 2003 The Anubis Team.
 # 
 #  GNU Anubis is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
