@@ -1,23 +1,19 @@
 /*
    This file is part of GNU Anubis 
-   Copyright (C) 2004 The Anubis Team.
+   Copyright (C) 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -26,8 +22,7 @@
 
 #include "headers.h"
 #include "extern.h"
-#include <getopt.h>
-#include <getline.h>
+#include "getopt.h"
 
 #define OPT_VERSION          257
 #define OPT_HELP             258
@@ -67,10 +62,10 @@ error (const char *fmt, ...)
   fprintf (stderr, "\n");
 }
 
-static void
-adm_memory_error (const char *msg)
+void
+xalloc_die ()
 {
-  error ("%s", msg);
+  error ("%s", _("Not enough memory"));
   exit (1);
 }
 
@@ -431,9 +426,6 @@ main (int argc, char **argv)
   else
     progname = argv[0];
 
-  /* Register memory error printer */
-  memory_error = adm_memory_error;
-
   /* Initialize various database formats */
 
   dbtext_init ();
