@@ -1,24 +1,20 @@
 /*
    anubisusr.c
    
-   Copyright (C) 2004 The Anubis Team.
+   Copyright (C) 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -28,8 +24,7 @@
 #include "extern.h"
 #include "rcfile.h"
 #include <gsasl.h>
-#include <getopt.h>
-#include <getline.h>
+#include "getopt.h"
 
 #define obstack_chunk_alloc malloc
 #define obstack_chunk_free free
@@ -164,41 +159,6 @@ skipword (char *str)
   return str;
 }
 
-int
-argcv_split (char *buf, int *pargc, char ***pargv)
-{
-  char *t;
-  int i, argc = 0;
-  char **argv;
-
-  t = buf;
-  do
-    {
-      argc++;
-      t = skipws (t);
-    }
-  while (*t && (t = skipword (t)));
-
-  argv = calloc (argc, sizeof (*argv));
-  for (i = 0, t = strtok (buf, " \t"); t; i++, t = strtok (NULL, " \t"))
-    argv[i] = strdup (t);
-  argv[i] = NULL;
-  *pargc = argc - 1;
-  *pargv = argv;
-  return 0;
-}
-
-void
-argcv_free (int argc, char **argv)
-{
-  if (argc == 0 || argv == NULL)
-    return;
-  while (--argc >= 0)
-    if (argv[argc])
-      free (argv[argc]);
-  free (argv);
-}
-
 /* FIXME: Move to the library and unify with hostname_error() */
 const char *
 h_error_string (int ec)
@@ -276,9 +236,14 @@ find_capa (struct smtp_reply *repl, const char *name, const char *value)
 	    {
 	      int j, argc;
 	      char **argv;
-	      int rc = 1;
+	      int rc = 0;
 
-	      argcv_split (repl->argv[i], &argc, &argv);
+	      if ((rc = argcv_get (repl->argv[i], "", NULL, &argc, &argv)))
+		{
+		  error (_("argcv_get failed: %s"), strerror (rc));
+		  return 1;
+		}
+	      rc = 1;
 	      for (j = 0; rc == 1 && j < argc; j++)
 		if (strcmp (argv[j], value) == 0)
 		  rc = 0;
@@ -306,11 +271,16 @@ find_capa_v (struct smtp_reply *repl, const char *name, ANUBIS_LIST * list)
       char *p = skipword (repl->argv[i]);
       if (strncmp (name, repl->argv[i], p - repl->argv[i]) == 0)
 	{
-	  int j, argc;
+	  int rc, j, argc;
 	  char **argv;
 	  char *rv = NULL;
 
-	  argcv_split (repl->argv[i], &argc, &argv);
+	  if ((rc = argcv_get (repl->argv[i], "", NULL, &argc, &argv)))
+	    {
+	      error (_("argcv_get failed: %s"), strerror (rc));
+	      return NULL;
+	    }
+
 	  if (!list)
 	    {
 	      if (argv[1])
@@ -568,6 +538,7 @@ parse_netrc (const char *filename)
 
   while (getline (&buf, &n, fp) > 0 && n > 0)
     {
+      int rc;
       char *p;
       size_t len;
       int argc;
@@ -581,8 +552,12 @@ parse_netrc (const char *filename)
       if (*p == 0 || *p == '#')
 	continue;
 
-      argcv_split (buf, &argc, &argv);
-
+      if ((rc = argcv_get (buf, "", "#", &argc, &argv)))
+	{
+	  error (_("argcv_get failed: %s"), strerror (rc));
+	  return;
+	}
+      
       if (strcmp (argv[0], "machine") == 0)
 	{
 	  if (hostcmp (argv[1], smtp_host) == 0)
@@ -640,7 +615,6 @@ parse_netrc (const char *filename)
     }
 }
 
-
 /* FIXME: Add UTF-8 conversion */
 static int
 utf8cpy (char *dst, size_t * dstlen, char *src, size_t srclen)
@@ -1237,6 +1211,12 @@ read_netrc (void)
     }
 }
 
+void
+xalloc_die ()
+{
+  error ("%s", _("Not enough memory"));
+  exit (1);
+}
 
 int
 main (int argc, char **argv)
@@ -1281,7 +1261,7 @@ main (int argc, char **argv)
 
 	case OPT_VERSION:
 	  printf ("anubisusr (%s)\n", PACKAGE_STRING);
-	  break;
+	  exit (0);
 
 	case OPT_HELP:
 	  help ();
