@@ -2,24 +2,20 @@
    authmode.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003, 2004 The Anubis Team.
+   Copyright (C) 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -286,7 +282,7 @@ asmtp_init (enum asmtp_state state)
   char *command = NULL;
   size_t s = 0;
 
-  recvline_ptr (SERVER, remote_client, &command, &s);
+  recvline (SERVER, remote_client, &command, &s);
 
   switch (asmtp_kw (get_command_word (command))) {
   case KW_EHLO:
@@ -335,7 +331,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
   char *mech;
   char *init_input;
 
-  if (recvline_ptr (SERVER, remote_client, &command, &s) <= 0)
+  if (recvline (SERVER, remote_client, &command, &s) <= 0)
     exit (1);
 
   switch (asmtp_kw (get_command_word (command))) {
@@ -352,7 +348,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
 	NET_STREAM stream;
 	
 	if (!secure.cert)
-	  secure.cert = allocbuf (DEFAULT_SSL_PEM, MAXPATHLEN);
+	  secure.cert = xstrdup (DEFAULT_SSL_PEM);
 	if (!check_filename (secure.cert, NULL))
 	  {
 	    asmtp_reply (454, "TLS not available due to temporary reason");
@@ -360,7 +356,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
 	  }
 
 	if (!secure.key)
-	  secure.key = allocbuf (secure.cert, MAXPATHLEN);
+	  secure.key = xstrdup (secure.cert);
 	else if (!check_filename (secure.key, NULL))
 	  {
 	    asmtp_reply (454, "TLS not available due to temporary reason");
@@ -390,7 +386,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
   case KW_AUTH:
     mech = get_command_arg ();
     init_input = get_command_arg ();
-    if (anubis_auth_gsasl (mech, init_input, usr, &remote_client) == 0)
+    if (anubis_auth_gsasl (mech, init_input, usr) == 0)
       state = state_auth;
     break;
     
@@ -469,7 +465,7 @@ xdb_loop (void)
   info (VERBOSE, _("Entering XDB loop..."));
 
   asmtp_capa_add ("XDATABASE");
-  while (recvline_ptr (SERVER, remote_client, &command, &s) > 0)
+  while (recvline (SERVER, remote_client, &command, &s) > 0)
     {
       switch (asmtp_kw (get_command_word (command))) {
       case KW_HELP:
@@ -558,13 +554,11 @@ anubis_get_db_record (const char *username, ANUBIS_USER * usr)
 
 
 int
-anubis_authenticate_mode (NET_STREAM *psd_client,
-			  struct sockaddr_in *addr)
+anubis_authenticate_mode (struct sockaddr_in *addr)
 {
   ANUBIS_USER usr;
 
-  remote_client = *psd_client;
-  remote_server = *psd_client;
+  remote_server = remote_client;
   alarm (900);
 
   if (anubis_smtp (&usr))
@@ -572,27 +566,22 @@ anubis_authenticate_mode (NET_STREAM *psd_client,
 
   if (usr.username)
     {
-      strncpy (session.clientname, usr.username, sizeof (session.clientname));
-      session.clientname[sizeof (session.clientname) - 1] = '\0';
-      if (check_username (session.clientname))
-	anubis_changeowner (session.clientname);
+      if (check_username (usr.username))
+	{
+	  anubis_changeowner (usr.username);
+	  assign_string (&session.clientname, usr.username);
+	}
       else
 	set_unprivileged_user ();
     }
   else
-    {
-      strncpy (session.clientname, usr.smtp_authid,
-	       sizeof (session.clientname));
-      session.clientname[sizeof (session.clientname) - 1] = '\0';
-      set_unprivileged_user ();
-    }
+    set_unprivileged_user ();
 
   if (usr.rcfile_name)
-    {
-      session.rcfile_name = usr.rcfile_name;
-      open_rcfile (CF_CLIENT);
-      process_rcfile (CF_CLIENT);
-    }
+    session.rcfile_name = usr.rcfile_name;
+
+  open_rcfile (CF_CLIENT);
+  process_rcfile (CF_CLIENT);
   
   if (topt & T_XELO)
     {
@@ -600,7 +589,7 @@ anubis_authenticate_mode (NET_STREAM *psd_client,
     }
   else
     {
-      if (!(topt & T_LOCAL_MTA) && strlen (session.mta) == 0)
+      if (!(topt & T_LOCAL_MTA) && !session.mta)
 	{
 	  anubis_error (EXIT_FAILURE, 0,
                         _("MTA has not been specified. "
@@ -678,7 +667,6 @@ anubis_authenticate_mode (NET_STREAM *psd_client,
   
   net_close_stream (&remote_client);
   net_close_stream (&remote_server);
-  *psd_client = NULL;
   
   info (NORMAL, _("Connection closed successfully."));
 
@@ -771,3 +759,4 @@ authmode_section_init (void)
 #endif /* WITH_GSASL */
 
 /* EOF */
+
