@@ -2,24 +2,20 @@
    daemon.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -32,10 +28,6 @@ int deny_severity = LOG_INFO;
 int allow_severity = LOG_INFO;
 #endif /* USE_LIBWRAP */
 
-static RETSIGTYPE sig_cld (int);
-
-static int nchild;
-
 /************
   DAEMONIZE
 *************/
@@ -105,26 +97,29 @@ format_exit_status (char *buffer, size_t buflen, int status)
   return buffer;
 }
 
-
-static RETSIGTYPE
-sig_cld (int code)
+static void
+report_process_status (size_t count, pid_t pid, int status)
 {
-  pid_t pid;
-  int status;
   char buffer[LINEBUFFER];
-  
-  while ((pid = waitpid (-1, &status, WNOHANG)) > 0)
-    {
-      nchild--;
-      info (VERBOSE,
-	    ngettext
-	    ("Child [%lu] finished. %s. %d client left.",
-	     "Child [%lu] finished. %s. %d clients left.",
-	     nchild), (unsigned long) pid,
-	    format_exit_status (buffer, sizeof buffer, status), nchild);
-    }
-  signal (code, sig_cld);
-  return;
+
+  count--;
+  info (VERBOSE,
+	ngettext
+	("Child [%lu] finished. %s. %d client left.",
+	 "Child [%lu] finished. %s. %d clients left.",
+	 count),
+	(unsigned long) pid,
+	format_exit_status (buffer, sizeof buffer, status), count);
+}
+
+static void
+subprocess_report_status (size_t count, pid_t pid, int status)
+{
+  char buffer[LINEBUFFER];
+
+  info (VERBOSE, _("Local program [%lu] finished. %s"),
+	(unsigned long) pid,
+	format_exit_status (buffer, sizeof buffer, status));
 }
 
 /************************************
@@ -157,13 +152,20 @@ set_unprivileged_user (void)
 {
   if (topt & T_USER_NOTPRIVIL)
     {
-      if (check_username (session.notprivileged))
+      if (check_username (session.notprivileged)) {
 	anubis_changeowner (session.notprivileged);
+	assign_string (&session.clientname, session.notprivileged);
+      }
+      else
+	anubis_error (EXIT_FAILURE, 0,
+		      _("WARNING: An unprivileged user cannot be resolved. Verify your settings!"));
     }
   else
     {
-      if (check_username (DEFAULT_UNPRIVILEGED_USER))
+      if (check_username (DEFAULT_UNPRIVILEGED_USER)) {
 	anubis_changeowner (DEFAULT_UNPRIVILEGED_USER);
+	assign_string (&session.clientname, DEFAULT_UNPRIVILEGED_USER);
+      }
       else
 	info (NORMAL,
 	      _("WARNING: An unprivileged user has not been specified!"));
@@ -172,24 +174,29 @@ set_unprivileged_user (void)
 }
 
 int
-anubis_child_main (NET_STREAM *sd_client, struct sockaddr_in *addr)
+anubis_child_main (struct sockaddr_in *addr)
 {
   int rc;
 
+  proclist_init ();
 #ifdef WITH_GSASL
   switch (anubis_mode)
     {
     case anubis_transparent:
-      rc = anubis_transparent_mode (sd_client, addr);
+      rc = anubis_transparent_mode (addr);
       break;
 
     case anubis_authenticate:
-      rc = anubis_authenticate_mode (sd_client, addr);
+      rc = anubis_authenticate_mode (addr);
+
+    default:
+      abort();
     }
 #else
-  rc = anubis_transparent_mode (sd_client, addr);
+  rc = anubis_transparent_mode (addr);
 #endif /* WITH_GSASL */
-  net_close_stream (sd_client);
+  proclist_cleanup (subprocess_report_status);
+  net_close_stream (&remote_client);
   return rc;
 }
 
@@ -206,17 +213,21 @@ loop (int sd_bind)
 #ifdef USE_LIBWRAP
   struct request_info req;
 #endif /* USE_LIBWRAP */
-  sigset_t blockset;
 
   addrlen = sizeof (addr);
-  signal (SIGCHLD, sig_cld);
+
+  proclist_init ();
 
   info (VERBOSE, _("GNU Anubis is running..."));
 
   for (;;)
     {
-      NET_STREAM sd_client = NULL;
-      int fd = accept (sd_bind, (struct sockaddr *) &addr, &addrlen);
+      int fd;
+      size_t count;
+      
+      fd = accept (sd_bind, (struct sockaddr *) &addr, &addrlen);
+      count = proclist_cleanup (report_process_status);
+      
       if (fd < 0)
 	{
 	  if (errno == EINTR)
@@ -229,8 +240,9 @@ loop (int sd_bind)
 	}
 
       /* Create the TCP stream */
-      net_create_stream (&sd_client, fd);
-
+      net_create_stream (&remote_client, fd);
+      remote_server = NULL;
+      
       /*
          Check the TCP wrappers settings.
        */
@@ -243,7 +255,7 @@ loop (int sd_bind)
 	  info (NORMAL,
 		_("TCP wrappers: connection from %s:%u rejected."),
 		inet_ntoa (addr.sin_addr), ntohs (addr.sin_port));
-	  service_unavailable (&sd_client);
+	  service_unavailable (&remote_client);
 	  continue;
 	}
 #endif /* USE_LIBWRAP */
@@ -258,18 +270,12 @@ loop (int sd_bind)
 	  process_rcfile (CF_SUPERVISOR);
 	}
 
-      sigemptyset (&blockset);
-      sigaddset (&blockset, SIGCHLD);
-      sigprocmask (SIG_BLOCK, &blockset, NULL);		
-      nchild++;
-      sigprocmask (SIG_UNBLOCK, &blockset, NULL);		
-      if (nchild > MAXCLIENTS)
+      if (count >= MAXCLIENTS)
 	{
 	  info (NORMAL,
 		_("Too many clients. Connection from %s:%u rejected."),
 		inet_ntoa (addr.sin_addr), ntohs (addr.sin_port));
-	  service_unavailable (&sd_client);
-	  nchild--;
+	  service_unavailable (&remote_client);
 	}
       else
 	{
@@ -283,12 +289,13 @@ loop (int sd_bind)
 	    {			/* a child process */
 	      /* FIXME */
 	      signal (SIGCHLD, SIG_IGN);
-	      quit (anubis_child_main (&sd_client, &addr));
+	      quit (anubis_child_main (&addr));
 	    }
-
-	  net_close_stream (&sd_client);
+	  else /* master process */
+	    proclist_register (childpid);
+	  
+	  net_close_stream (&remote_client);
 	}
-      cleanup_children ();
     }
   return;
 }
@@ -299,7 +306,7 @@ loop (int sd_bind)
 *********************************************/
 
 static int
-_stdio_write (void *sd, char *data, size_t size, size_t * nbytes)
+_stdio_write (void *sd, const char *data, size_t size, size_t * nbytes)
 {
   int rc;
   int fd = (int) sd;
@@ -333,8 +340,10 @@ _stdio_read (void *sd, char *data, size_t size, size_t * nbytes)
       n = read (fd, data, size);
       if (n >= 0)
 	*nbytes = n;
+      else
+	return errno;
     }
-  return errno;
+  return 0;
 }
 
 static const char *
@@ -343,55 +352,57 @@ _stdio_strerror (void *ignored_data, int rc)
   return strerror (rc);
 }
 
+void
+create_stdio_stream (NET_STREAM *s)
+{
+  net_create_stream (s, 0);
+  stream_set_read (*s, _stdio_read);
+  stream_set_write (*s, _stdio_write);
+  stream_set_strerror (*s, _stdio_strerror);
+}
+
 void
 stdinout (void)
 {
-  NET_STREAM sd_client = NULL;
-  NET_STREAM sd_server = NULL;
-
   topt &= ~T_SSL;
   topt |= T_FOREGROUND;
   topt |= T_SMTP_ERROR_CODES;
 
-  anubis_getlogin (session.clientname, sizeof (session.clientname));
+  proclist_init ();
+  
+  anubis_getlogin (&session.clientname);
   auth_tunnel ();		/* session.clientname = session.supervisor */
 
-  if (!(topt & T_LOCAL_MTA) && (strlen (session.mta) == 0))
+  if (!(topt & T_LOCAL_MTA) && !session.mta)
     {
       options.termlevel = NORMAL;
       anubis_error (EXIT_FAILURE, 0, _("The MTA has not been specified. "
 			               "Set the REMOTE-MTA or LOCAL-MTA."));
     }
 
-  net_create_stream (&sd_client, 0);
-  stream_set_read (sd_client, _stdio_read);
-  stream_set_write (sd_client, _stdio_write);
-  stream_set_strerror (sd_client, _stdio_strerror);
+  create_stdio_stream (&remote_client);
 
   alarm (300);
   if (topt & T_LOCAL_MTA)
-    sd_server = make_local_connection (session.execpath, session.execargs);
+    remote_server = make_local_connection (session.execpath, session.execargs);
   else
-    sd_server = make_remote_connection (session.mta, session.mta_port);
+    remote_server = make_remote_connection (session.mta, session.mta_port);
   alarm (0);
 
-  if (sd_server == NULL)
+  if (remote_server == NULL)
     {
-      service_unavailable (&sd_client);
+      service_unavailable (&remote_client);
       free_mem ();
       return;
     }
-  stream_set_read (sd_server, _stdio_read);
-  stream_set_write (sd_server, _stdio_write);
-  stream_set_strerror (sd_server, _stdio_strerror);
-
-  remote_client = sd_client;
-  remote_server = sd_server;
+  stream_set_read (remote_server, _stdio_read);
+  stream_set_write (remote_server, _stdio_write);
+  stream_set_strerror (remote_server, _stdio_strerror);
 
   smtp_session_transparent ();
-  cleanup_children ();
-
-  net_close_stream (&sd_server);
+  proclist_cleanup (subprocess_report_status);
+  
+  net_close_stream (&remote_server);
   free_mem ();
   return;
 }
