@@ -2,24 +2,20 @@
    esmtp.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -151,12 +147,13 @@ cb_realm (Gsasl_session_ctx *ctx, char *out, size_t *outlen)
 }
 
 static char *
-get_reply (NET_STREAM str, int *code, char *buf, size_t size)
+get_reply (NET_STREAM str, int *code, char **buf, size_t *psize)
 {
   char *p;
-  get_response_smtp (CLIENT, str, buf, size);
-  remcrlf (buf);
-  *code = strtoul (buf, &p, 10);
+  
+  get_response_smtp (CLIENT, str, buf, psize);
+  remcrlf (*buf);
+  *code = strtoul (*buf, &p, 10);
   if (*p == 0 || *p == '\r')
     return p;
 
@@ -176,12 +173,14 @@ do_gsasl_auth (NET_STREAM *pstr, Gsasl_ctx * ctx, char *mech)
   char *output;
   int rc;
   Gsasl_session_ctx *sess_ctx = NULL;
-  char buf[LINEBUFFER + 1];
+  char sbuf[LINEBUFFER + 1];
+  char *buf = NULL;
+  size_t size = 0;
   char *p;
   int code;
   
-  snprintf (buf, sizeof buf, "AUTH %s" CRLF, mech);
-  swrite (CLIENT, *pstr, buf);
+  snprintf (sbuf, sizeof sbuf, "AUTH %s" CRLF, mech);
+  swrite (CLIENT, *pstr, sbuf);
 
   rc = gsasl_client_start (ctx, mech, &sess_ctx);
   if (rc != GSASL_OK)
@@ -192,10 +191,11 @@ do_gsasl_auth (NET_STREAM *pstr, Gsasl_ctx * ctx, char *mech)
 
   output = NULL;
 
-  p = get_reply (*pstr, &code, buf, sizeof buf);
+  p = get_reply (*pstr, &code, &buf, &size);
   if (code != 334)
     {
       anubis_error (0, 0, _("GSASL handshake aborted: %d %s"), code, p);
+      free (buf);
       return 1;
     }
 
@@ -210,30 +210,31 @@ do_gsasl_auth (NET_STREAM *pstr, Gsasl_ctx * ctx, char *mech)
 
       if (rc == GSASL_OK)
 	break;
-      p = get_reply (*pstr, &code, buf, sizeof buf);
+      p = get_reply (*pstr, &code, &buf, &size);
       if (code != 334)
 	{
 	  anubis_error (0, 0, _("GSASL handshake aborted: %d %s"), code, p);
 	  free (output);
+	  free (buf);
 	  return 1;
 	}
     }
   while (rc == GSASL_NEEDS_MORE);
 
   free (output);
-
+  
   if (rc != GSASL_OK)
     {
       anubis_error (0, 0, _("GSASL error: %s"), gsasl_strerror (rc));
       exit (1);
     }
 
-  p = get_reply (*pstr, &code, buf, sizeof buf);
+  p = get_reply (*pstr, &code, &buf, &size);
   
   if (code == 334)
     {
       /* Additional data. Do we need it? */
-      p = get_reply (*pstr, &code, buf, sizeof buf);
+      p = get_reply (*pstr, &code, &buf, &size);
     }
 
   if (code != 235)
@@ -246,6 +247,8 @@ do_gsasl_auth (NET_STREAM *pstr, Gsasl_ctx * ctx, char *mech)
   if (sess_ctx)
     install_gsasl_stream (sess_ctx, pstr);
 
+  free (buf);
+  
   return 0;
 }
 
