@@ -2,68 +2,26 @@
    exec.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
 #include "extern.h"
 
 static int make_sockets (int fd[]);
-static void sig_local (int);
-
-/********************************
- execargs generator for execvp()
-*********************************/
-
-char **
-gen_execargs (const char *commandline)
-{
-  int argc;
-  char *dupstr;
-  char *str;
-  char **args;
-
-  /* strtok modifies the string. Prevent it by making a private copy. */
-  dupstr = strdup (commandline);
-
-  /* Count the arguments first. */
-  for (argc = 0, str = strtok (dupstr, " "); str; argc++)
-    str = strtok (0, " ");
-  free (dupstr);
-
-  args = xmalloc ((argc + 1) * sizeof (*args));
-
-  /* strtok above has modified the string, so we duplicate it again. */
-  dupstr = strdup (commandline);
-
-  /* Now copy the arguments. */
-  for (argc = 0, str = strtok (dupstr, " "); str; argc++)
-    {
-      args[argc] = strdup (str);
-      str = strtok (0, " ");
-    }
-  args[argc] = NULL;
-  free (dupstr);
-
-  return args;
-}
 
 /*************************
  Connect to stdin/stdout.
@@ -133,55 +91,28 @@ make_sockets (int fd[2])
   return 0;
 }
 
-void
-cleanup_children ()
-{
-  pid_t pid;
-  int status;
-
-  while ((pid = waitpid (-1, &status, WNOHANG)) > 0)
-    info (VERBOSE, _("Local program [%lu] finished."), (unsigned long) pid);
-}
-
-static RETSIGTYPE
-sig_local (int code)
-{
-  /* EMPTY FUNCTION */
-  /* Notice: This signal handler is installed only by
-     make_local_connection, that is used to launch the local
-     mailer. After the mailer exits, the main code will read
-     the return code from the mailer's outgoing pipe. If
-     waitpid() is called before this happens, the pipe will be
-     broken and the read will fail. Therefore, do not call waitpid()
-     from the handler. This should be done after smtp_session()
-     by calling cleanup_children() */
-}
-
 static int
 make_local_connection_fd (char *exec_path, char **exec_args)
 {
   int fd[2];
-  char **pargs;
-  char args[LINEBUFFER + 1];
-
+  pid_t pid;
+  
   if (check_filename (exec_path, 0) == 0)
     return -1;
 
-  memset (args, 0, LINEBUFFER + 1);
-  pargs = exec_args;
-  pargs++;
-  while (*pargs)
+  if (VERBOSE > options.termlevel) /* Extra check to avoid unnecessary
+				      memory allocation */
     {
-      strncat (args, *pargs, LINEBUFFER - strlen (args));
-      strncat (args, " ", LINEBUFFER - strlen (args));
-      pargs++;
+        char *args;
+	argcv_string (-1, exec_args, &args);
+	info (VERBOSE, _("Executing %s..."), args);
+	free (args);
     }
-  info (VERBOSE, _("Executing %s %s..."), exec_path, args);
+  
   if (make_sockets (fd))
     return -1;
 
-  signal (SIGCHLD, sig_local);
-  switch (fork ())
+  switch (pid = fork ())
     {
     case -1:			/* an error */
       anubis_error (0, errno, _("fork() failed"));
@@ -199,6 +130,10 @@ make_local_connection_fd (char *exec_path, char **exec_args)
 	close (fd[1]);
       execvp (exec_path, exec_args);
       anubis_error (EXIT_FAILURE, errno, _("execvp() failed"));
+
+    default:
+      /* Master: register created process */
+      proclist_register (pid);
     }
   close (fd[1]);
 #ifdef FD_CLOEXEC
@@ -219,7 +154,7 @@ make_local_connection (char *exec_path, char **exec_args)
   net_create_stream (&str, fd);
   return str;
 }
-
+  
 /*************************************
  Use an external program, which works
  on standard input and output.
@@ -228,38 +163,16 @@ make_local_connection (char *exec_path, char **exec_args)
 char *
 external_program (int *rs, char *path, char *src, char *dst, int dstsize)
 {
+  int rc;
   char *ret;
-  char tmp[LINEBUFFER + 1];
-  char **args = 0;
-  char *a = 0;			/* args */
-  char *p = 0;			/* path */
+  int argc;
+  char **argv = 0;
 
-  a = strchr (path, ' ');	/* an extra arguments */
-  if (a)
-    {
-      *a++ = '\0';
-      p = strrchr (path, '/');
-      if (p)
-	p++;
-      else
-	p = path;
-      snprintf (tmp, LINEBUFFER, "%s %s", p, a);
-      p = path;
-      a = tmp;
-    }
-  else
-    {				/* no arguments */
-      p = path;
-      a = strrchr (path, '/');
-      if (a)
-	a++;
-      else
-	a = path;
-    }
-
-  args = gen_execargs (a);
-  ret = exec_argv (rs, p, args, src, dst, dstsize);
-  xfree_pptr (args);
+  if ((rc = argcv_get (path, "", "#", &argc, &argv)))
+    anubis_error (EX_SOFTWARE, rc, _("argcv_get failed"));
+  
+  ret = exec_argv (rs, argv[0], argv, src, dst, dstsize);
+  argcv_free (argc, argv);
   return ret;
 }
 
@@ -271,7 +184,8 @@ exec_argv (int *rs, char *path, char **argv, char *src, char *dst,
   int fd;
   int n;
   char *buf;
-
+  size_t dstpos;
+  
   fd = make_local_connection_fd (path ? path : argv[0], argv);
   if (fd == -1)
     {
@@ -292,38 +206,46 @@ exec_argv (int *rs, char *path, char **argv, char *src, char *dst,
 
   buf = (char *) xmalloc (DATABUFFER + 1);
   memset (dst, 0, dstsize);
+  dstpos = 0;
 
   if (dst && dstsize)
     {				/* static array */
-      while ((n = read (fd, buf, DATABUFFER)) > 0)
+      dstsize--; /* leave place for the terminating nul */
+      while (dstpos < dstsize && (n = read (fd, buf, DATABUFFER)) > 0)
 	{
-	  strncat (dst, buf, dstsize);
-	  memset (buf, 0, DATABUFFER + 1);
-	  dstsize -= n;
-	  if (dstsize < 1)
-	    break;
+	  size_t len = dstsize - dstpos;
+	  if (len > n)
+	    len = n;
+	  memcpy (dst + dstpos, buf, len);
+	  dstpos += len;
 	}
+      dst[dstpos] = 0;   
     }
   else
     {				/* dynamic array */
-      dst = (char *) xmalloc (1);
+      dstsize = DATABUFFER;
+      dst = xmalloc (dstsize);
       while ((n = read (fd, buf, DATABUFFER)) > 0)
 	{
-	  dst = xrealloc (dst, strlen (dst) + n + 1);
-	  strncat (dst, buf, n);
-	  memset (buf, 0, DATABUFFER + 1);
+	  if (dstsize - dstpos < n)
+	    {
+	       dstsize += DATABUFFER;
+	       dst = xrealloc (dst, dstsize);
+	    } 
+	  memcpy (dst + dstpos, buf, n);  
 	}
-      free (buf);
-      close (fd);
-      waitpid (-1, &status, WNOHANG);
-      *rs = 0;
-      return dst;
+      if (dstsize - dstpos < n)
+        {
+          dstsize++;
+          dst = xrealloc (dst, dstsize);
+        }
+      dst[dstpos] = 0;
     }
   free (buf);
   close (fd);
   waitpid (-1, &status, WNOHANG);
   *rs = 0;
-  return 0;
+  return dst;
 }
 
 /* EOF */
