@@ -2,24 +2,20 @@
    gpg.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -49,7 +45,8 @@ static char *gpg_sign_encrypt (char *);
 static void gpgme_debug_info (gpgme_ctx_t);
 
 #define EXTRA_GPG_BUF 4096
-#define fail_if_err(a) do { \
+#define fail_if_err(code) do { \
+		int a = code;\
 		if (a) { \
 			anubis_error(EXIT_FAILURE, 0, _("GPGME: %s."), \
 			             gpgme_strerror(a)); \
@@ -118,12 +115,53 @@ gpgme_init (void)
   return 0;
 }
 
-static const char *
-passphrase_cb (void *hook, const char *desc, void **r_hd)
+gpgme_error_t
+passphrase_cb (void *hook, const char *uid_hint, const char *passphrase_info, 
+	       int prev_was_bad, int fd)
 {
-  if (!desc)
+  if (!passphrase_info)
     return 0;
-  return gpg.passphrase;
+  
+  size_t len = strlen(gpg.passphrase);
+  if (write (fd, gpg.passphrase, len) != len)
+    return gpg_error(GPG_ERR_CANCELED);
+  if (write (fd, "\n", 1) != 1)
+    return gpg_error(GPG_ERR_CANCELED);
+        
+  return 0;
+}
+
+static int
+anubis_gpg_read (gpgme_data_t dh, size_t size, char **pdata)
+{
+  char buf[256];
+  char *data;
+  size_t pos;
+  size_t nread;
+  
+  pos = 0;
+  size += EXTRA_GPG_BUF;
+  data = xmalloc (size);
+  while ((nread = gpgme_data_read (dh, buf, sizeof (buf))) > 0)
+    {
+      if (size - pos < nread)
+	{
+	  size += sizeof (buf);
+	  data = xrealloc (data, size);
+	}
+      memcpy (data + pos, buf, nread);
+      pos += nread;
+    }
+  if (size - pos == 0)
+    {
+      size++;
+      data = xrealloc (data, size);
+    }
+  data[pos] = 0;
+  if (nread == -1)
+    fail_if_err (errno);
+  *pdata = data;
+  return pos ;
 }
 
 static char *
@@ -133,14 +171,8 @@ gpg_sign (char *gpg_data)
   gpgme_error_t err = 0;
   gpgme_data_t in, out;
   gpgme_key_t key;
-  char buf[256];
   char *p, *signed_data;
-  int size;
-  size_t nread;
 
-  size = strlen (gpg_data) + EXTRA_GPG_BUF;
-  signed_data = (char *) xmalloc (size);
-  memset (buf, 0, sizeof (buf));
   fail_if_err (gpgme_new (&ctx));
 
   if (gpg.sign_keys)
@@ -158,7 +190,6 @@ gpg_sign (char *gpg_data)
 	{
 	  anubis_error (0, 0, _("GPGME: Cannot list keys: %s"),
 			gpgme_strerror (err));
-          xfree (signed_data);
           gpgme_release (ctx);
           return NULL;
 	}
@@ -166,7 +197,7 @@ gpg_sign (char *gpg_data)
 
   p = getenv ("GPG_AGENT_INFO");
   if (!(p && strchr (p, ':')))
-    gpgme_set_passphrase_cb (ctx, (gpgme_passphrase_cb_t) passphrase_cb, 0);
+    gpgme_set_passphrase_cb (ctx, passphrase_cb, 0);
   gpgme_set_textmode (ctx, 1);
   gpgme_set_armor (ctx, 1);
 
@@ -178,26 +209,8 @@ gpg_sign (char *gpg_data)
   if (options.termlevel == DEBUG)
     gpgme_debug_info (ctx);
 
-  while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
-    {
-      if (size > nread)
-	{
-	  strncat (signed_data, buf, nread);
-	  size -= nread;
-	}
-      else
-	{
-	  size = EXTRA_GPG_BUF;
-	  signed_data = (char *) xrealloc ((char *) signed_data,
-					   strlen (signed_data) + size);
-	  strncat (signed_data, buf, nread);
-	  size -= nread;
-	}
-      memset (buf, 0, sizeof (buf));
-    }
-  if (nread == -1)
-    fail_if_err (errno);
-
+  anubis_gpg_read (out, strlen (gpg_data), &signed_data);
+  
   gpgme_data_release (in);
   gpgme_data_release (out);
   gpgme_release (ctx);
@@ -260,18 +273,12 @@ gpg_encrypt (char *gpg_data)
 {
   gpgme_ctx_t ctx;
   gpgme_data_t in, out;
-  char buf[256];
   char *encrypted_data;
   size_t nread;
   gpgme_key_t *keyptr;
   struct obstack stk;
   gpgme_encrypt_result_t result;
-  size_t size;
   
-  size = strlen (gpg_data) + EXTRA_GPG_BUF;
-  encrypted_data = (char *) xmalloc (size);
-  memset (buf, 0, sizeof (buf));
-
   fail_if_err (gpgme_new (&ctx));
   gpgme_set_armor (ctx, 1);
 
@@ -293,23 +300,7 @@ gpg_encrypt (char *gpg_data)
   if (options.termlevel == DEBUG)
     gpgme_debug_info (ctx);
 
-  while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
-    {
-      if (size > nread)
-	{
-	  strncat (encrypted_data, buf, nread);
-	  size -= nread;
-	}
-      else
-	{
-	  size = EXTRA_GPG_BUF;
-	  encrypted_data = (char *) xrealloc ((char *) encrypted_data,
-					      strlen (encrypted_data) + size);
-	  strncat (encrypted_data, buf, nread);
-	  size -= nread;
-	}
-      memset (buf, 0, sizeof (buf));
-    }
+  anubis_gpg_read (out, strlen (gpg_data), &encrypted_data);
   for (; *keyptr; keyptr++)
     gpgme_key_unref (*keyptr);
   obstack_free (&stk, NULL);
@@ -378,17 +369,11 @@ gpg_sign_encrypt (char *gpg_data)
   gpgme_error_t err = 0;
   gpgme_data_t in, out;
   gpgme_key_t *keyptr, key;
-  char buf[256];
   char *p, *se_data;		/* Signed-Encrypted Data */
-  int size;
   gpgme_encrypt_result_t result;
   gpgme_sign_result_t sign_result;
   struct obstack stk;
   
-  size = strlen (gpg_data) + EXTRA_GPG_BUF;
-  se_data = (char *) xmalloc (size);
-  memset (buf, 0, sizeof (buf));
-
   fail_if_err (gpgme_new (&ctx));
 
   if (gpg.sign_keys)
@@ -432,31 +417,12 @@ gpg_sign_encrypt (char *gpg_data)
   sign_result = gpgme_op_sign_result (ctx);
   if (check_result (sign_result, GPGME_SIG_MODE_NORMAL) == 0)
     {
-      size_t nread;
       fail_if_err (rewind_gpgme_data (out));
 
       if (options.termlevel == DEBUG)
         gpgme_debug_info (ctx);
 
-      while ((nread = gpgme_data_read (out, buf, sizeof (buf))) > 0)
-        {
-          if (size > nread)
-	    {
-	      strncat (se_data, buf, nread);
-	      size -= nread;
- 	    }
-          else
-	    {
-	      size = EXTRA_GPG_BUF;
-	      se_data = (char *) xrealloc ((char *) se_data,
-				           strlen (se_data) + size);
-	      strncat (se_data, buf, nread);
-	      size -= nread;
-	    }
-          memset (buf, 0, sizeof (buf));
-        }
-     if (nread == -1)
-       fail_if_err (errno);
+      anubis_gpg_read (out, strlen (gpg_data), &se_data);
     }
   else 
     xfree (se_data);
@@ -529,7 +495,7 @@ gpg_parser (int method, int key, ANUBIS_LIST * arglist,
 
     case KW_GPG_ENCRYPT:
       xfree (gpg.encryption_keys);
-      gpg.encryption_keys = allocbuf (arg, 0);
+      gpg.encryption_keys = xstrdup (arg);
       if (gpg.inited == 0 && gpgme_init ())
 	break;
       gpg_proc (msg, gpg_encrypt);
@@ -557,13 +523,13 @@ gpg_parser (int method, int key, ANUBIS_LIST * arglist,
 	    if (strcasecmp (p, "default") && strcasecmp (p, "yes"))
 	      {
 		xfree (gpg.sign_keys);
-		gpg.sign_keys = allocbuf (p, 0);
+		gpg.sign_keys = xstrdup (p);
 	      }
 	    *--p = '\0';
-	    gpg.encryption_keys = allocbuf (arg, 0);
+	    gpg.encryption_keys = xstrdup (arg);
 	  }
 	else
-	  gpg.encryption_keys = allocbuf (arg, 0);
+	  gpg.encryption_keys = xstrdup (arg);
 
 	if (gpg.inited == 0 && gpgme_init ())
 	  break;
