@@ -2,24 +2,20 @@
    gsasl_srv.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003,2004 The Anubis Team.
+   Copyright (C) 2003, 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -90,12 +86,6 @@ anubis_set_server_mech_list (ANUBIS_LIST *list)
 
 /* Capability list handling */
 
-int
-anubis_name_cmp (void *item, void *data)
-{
-  return strcmp (item, data);
-}
-
 static void
 auth_gsasl_capa_init ()
 {
@@ -113,7 +103,12 @@ auth_gsasl_capa_init ()
     {
       size_t size = strlen (listmech);
       ANUBIS_LIST *mech = auth_method_list (listmech);
-      ANUBIS_LIST *p = list_intersect (mech, anubis_mech_list,
+
+      /* Compute intersection of both lists. Make sure we compute
+	 (anubis_mech_list X mech), not (mech X anubis_mech_list),
+	 so that the resulting list preserves the ordering of
+	 anubis_mech_list */
+      ANUBIS_LIST *p = list_intersect (anubis_mech_list, mech,
 				       anubis_name_cmp);
       auth_list_to_string (p, listmech, size);
       list_destroy (&p, NULL, NULL);
@@ -132,8 +127,7 @@ auth_gsasl_capa_init ()
 #define SP(x) ((x)?(x):"NULL")
 
 int
-anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr,
-		   NET_STREAM * stream)
+anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr)
 {
   char *input = arg;
   size_t input_size = 0;
@@ -165,7 +159,7 @@ anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr,
   while ((rc = gsasl_step64 (sess_ctx, input, &output)) == GSASL_NEEDS_MORE)
     {
       asmtp_reply (334, "%s", output);
-      recvline_ptr (SERVER, remote_client, &input, &input_size);
+      recvline (SERVER, remote_client, &input, &input_size);
       remcrlf (input);
       if (strcmp (input, "*") == 0)
 	{
@@ -202,7 +196,7 @@ anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr,
 	usr->smtp_authid, usr->username ? usr->username : "NONE");
 
   if (sess_ctx)
-    install_gsasl_stream (sess_ctx, stream);
+    install_gsasl_stream (sess_ctx, &remote_client);
 
   asmtp_reply (235, "Authentication successful.");
   return 0;
