@@ -2,24 +2,20 @@
    guile.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003, 2004 The Anubis Team.
+   Copyright (C) 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -84,8 +80,11 @@ guile_ports_open (void)
     name = "/dev/null";
 
   if (fd == -1)
-    fd = open ("/dev/null", O_WRONLY);
-
+    {
+      name = "/dev/null";
+      fd = open (name, O_WRONLY|O_APPEND);
+    }
+  
   port = scm_fdes_to_port (fd, "a", scm_makfrom0str (name));
   guile_ports_close ();
   scm_set_current_error_port (port);
@@ -103,16 +102,22 @@ guile_ports_close (void)
 }
 
 void
-guile_load_path_append (char *path)
+guile_load_path_append (ANUBIS_LIST *arglist, MESSAGE *msg)
 {
+  char *path = list_item (arglist, 0);
   SCM scm, path_scm, *pscm;
   path_scm = SCM_VARIABLE_REF (scm_c_lookup ("%load-path"));
   for (scm = path_scm; scm != SCM_EOL; scm = SCM_CDR (scm))
     {
       SCM val = SCM_CAR (scm);
-      if (SCM_NIMP (val) && SCM_STRINGP (val))
-	if (strcmp (SCM_STRING_CHARS (val), path) == 0)
-	  return;
+      if (scm_is_string (val))
+	{
+	  char *p = scm_to_locale_string (val);
+	  int rc = strcmp (p, path);
+	  free (p);
+	  if (rc == 0)
+	    return;
+	}
     }
 
   pscm = SCM_VARIABLE_LOC (scm_c_lookup ("%load-path"));
@@ -122,17 +127,9 @@ guile_load_path_append (char *path)
 }
 
 void
-guile_load_program (char *filename)
+guile_load_program (ANUBIS_LIST *arglist, MESSAGE *msg)
 {
-  guile_ports_open ();
-  scm_primitive_load_path (scm_makfrom0str (filename));
-  guile_ports_close ();
-}
-
-static SCM
-eval_catch_body (void *list)
-{
-  return scm_primitive_eval ((SCM) list);
+  scm_primitive_load_path (scm_makfrom0str (list_item (arglist, 0)));
 }
 
 static SCM
@@ -156,8 +153,8 @@ guile_to_anubis (SCM cell)
 	{
 	  ASSOC *asc = xmalloc (sizeof (*asc));
 
-	  asc->key = SCM_STRING_CHARS (SCM_CAR (car));
-	  asc->value = SCM_STRING_CHARS (SCM_CDR (car));
+	  asc->key = scm_to_locale_string (SCM_CAR (car));
+	  asc->value = scm_to_locale_string (SCM_CDR (car));
 	  list_append (list, asc);
 	}
     }
@@ -169,8 +166,7 @@ anubis_to_guile (ANUBIS_LIST * list)
 {
   ASSOC *asc;
   ITERATOR *itr;
-  SCM head = SCM_EOL, tail;	/* Don't let gcc fool you: tail cannot be used 
-				   uninitialized */
+  SCM head = SCM_EOL, tail = SCM_EOL;
 
   itr = iterator_create (list);
   for (asc = iterator_first (itr); asc; asc = iterator_next (itr))
@@ -184,18 +180,15 @@ anubis_to_guile (ANUBIS_LIST * list)
 
       cdr = scm_makfrom0str (asc->value);
 
-      SCM_NEWCELL (cell);
-
-      SCM_SETCAR (cell, scm_cons (car, cdr));
+      cell = scm_cons (scm_cons (car, cdr), SCM_EOL);
       if (head == SCM_EOL)
 	head = cell;
       else
 	SCM_SETCDR (tail, cell);
+
       tail = cell;
     }
   iterator_destroy (&itr);
-  if (head != SCM_EOL)
-    SCM_SETCDR (tail, SCM_EOL);
   return head;
 }
 
@@ -213,7 +206,6 @@ list_to_args (ANUBIS_LIST * arglist)
   while ((p = iterator_next (itr)))
     {
       SCM cell;
-      SCM_NEWCELL (cell);
 
       if (p[0] == '#')
 	{
@@ -234,7 +226,7 @@ list_to_args (ANUBIS_LIST * arglist)
       else
 	val = scm_makfrom0str (p);
 
-      SCM_SETCAR (cell, scm_list_2 (SCM_IM_QUOTE, val));
+      cell = scm_cons (val, SCM_EOL);
 
       if (head == SCM_EOL)
 	head = cell;
@@ -243,21 +235,18 @@ list_to_args (ANUBIS_LIST * arglist)
       tail = cell;
     }
   iterator_destroy (&itr);
-  if (head != SCM_EOL)
-    SCM_SETCDR (tail, SCM_EOL);
   return head;
 }
 
 /* (define (postproc header-list body) */
 
 void
-guile_process_proc (ANUBIS_LIST * arglist, MESSAGE * msg)
+guile_process_proc (ANUBIS_LIST *arglist, MESSAGE *msg)
 {
   char *procname;
   SCM arg_hdr, arg_body;
   SCM invlist, rest_arg;
   SCM procsym;
-  jmp_buf jmp_env;
   SCM res;
 
   procname = list_item (arglist, 0);
@@ -282,23 +271,14 @@ guile_process_proc (ANUBIS_LIST * arglist, MESSAGE * msg)
       return;
     }
 
-  guile_ports_open ();
-  if (setjmp (jmp_env))
-    {
-      guile_ports_close ();
-      return;
-    }
-
-  invlist = scm_append (SCM_LIST2 (SCM_LIST3 (procsym,
-					      SCM_LIST2 (SCM_IM_QUOTE,
-							 arg_hdr),
-					      SCM_LIST2 (SCM_IM_QUOTE,
-							 arg_body)),
-				   rest_arg));
-  res = scm_internal_lazy_catch (SCM_BOOL_T,
-				 eval_catch_body,
-				 (void *) invlist,
-				 eval_catch_handler, &jmp_env);
+  invlist = scm_append
+               (scm_list_2
+		  (scm_list_3 (procsym,
+			       scm_cons (SCM_IM_QUOTE, arg_hdr),
+			       arg_body),
+		   rest_arg));
+  
+  res = scm_primitive_eval (invlist);
 
   if (SCM_IMP (res) && SCM_BOOLP (res))
     {
@@ -332,18 +312,17 @@ guile_process_proc (ANUBIS_LIST * arglist, MESSAGE * msg)
 	  free (msg->body);
 	  msg->body = strdup ("");
 	}
-      else if (SCM_NIMP (ret_body) && SCM_STRINGP (ret_body))
+      else if (scm_is_string (ret_body))
 	{
 	  /* Replace with the given string */
 	  xfree (msg->body);
-	  msg->body = strdup (SCM_STRING_CHARS (ret_body));
+	  msg->body = scm_to_locale_string (ret_body);
 	}
       else
 	anubis_error (0, 0, _("Bad cdr type in return from %s"), procname);
     }
   else
     anubis_error (0, 0, _("Bad return type from %s"), procname);
-  guile_ports_close ();
 }
 
 /* RC file stuff */
@@ -374,43 +353,77 @@ static struct rc_kwdef guile_rule_kw[] = {
   {NULL}
 };
 
+struct inner_closure
+{
+  ANUBIS_LIST *arglist;
+  MESSAGE *msg;
+  void (*fun) (ANUBIS_LIST *arglist, MESSAGE *msg);
+};
+
+static SCM
+inner_catch_body (void *data)
+{
+  struct inner_closure *closure = data;
+  closure->fun (closure->arglist, closure->msg);
+  return SCM_BOOL_F;
+}
+
 int
 guile_parser (int method, int key, ANUBIS_LIST * arglist,
 	      void *inv_data, void *func_data, MESSAGE * msg)
 {
+  int rc;
   char *arg = list_item (arglist, 0);
+  struct inner_closure closure;
+  jmp_buf jmp_env;
+  
+  closure.arglist = arglist;
+  closure.msg = msg;
+
   switch (key)
     {
     case KW_GUILE_OUTPUT:
       xfree (options.glogfile);
       options.glogfile = strdup (arg);
-      break;
+      return RC_KW_HANDLED;
 
     case KW_GUILE_DEBUG:
       guile_debug (strncmp ("yes", arg, 3) == 0);
-      break;
+      return RC_KW_HANDLED;
 
     case KW_GUILE_LOAD_PATH_APPEND:
-      guile_load_path_append (arg);
+      closure.fun = guile_load_path_append;
       break;
 
     case KW_GUILE_LOAD_PROGRAM:
-      guile_load_program (arg);
+      closure.fun = guile_load_program;
       break;
 
     case KW_GUILE_PROCESS:
-      guile_process_proc (arglist, msg);
+      closure.fun = guile_process_proc;
       break;
 
     case KW_GUILE_REWRITE_LINE:
-       /*FIXME*/
-	/*guile_rewrite_line(arg, line); */
-	break;
+      /*FIXME*/
+      /*guile_rewrite_line(arg, line); */
+      return RC_KW_HANDLED;
 
     default:
       return RC_KW_UNKNOWN;
     }
-  return RC_KW_HANDLED;
+
+  guile_ports_open ();
+  if (setjmp (jmp_env) == 0)
+    scm_internal_lazy_catch (SCM_BOOL_T,
+			     inner_catch_body,
+			     &closure,
+			     eval_catch_handler, &jmp_env);
+  else
+    rc = RC_KW_ERROR;
+
+  guile_ports_close ();
+
+  return rc;
 }
 
 static struct rc_secdef_child guile_secdef_child = {
