@@ -2,38 +2,38 @@
    headers.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
 
 #include <config.h>
 
+#if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
+#  define __attribute__(x)
+#endif
+
+#ifndef ANUBIS_PRINTFLIKE
+# define ANUBIS_PRINTFLIKE(fmt,narg) \
+  __attribute__ ((__format__ (__printf__, fmt, narg)))
+#endif
+
 # if defined(HAVE_GNUTLS_GNUTLS_H)
 #  define HAVE_GNUTLS
 # endif	/* HAVE_GNUTLS_GNUTLS_H */
-# if defined(HAVE_LIBSSL) && defined(HAVE_LIBCRYPTO)
-#  if defined(HAVE_OPENSSL_SSL_H)
-#   define HAVE_OPENSSL
-#  endif /* HAVE_OPENSSL_SSL_H */
-# endif	/* HAVE_LIBSSL and HAVE_LIBCRYPTO */
 # if defined(HAVE_LIBGPGME) && defined(HAVE_GPGME_H) && !defined(NOGPG)
 #  define HAVE_GPG
 # endif	/* HAVE_LIBGPGME and HAVE_GPGME_H and not NOGPG */
@@ -107,20 +107,8 @@
 # include <gnutls/gnutls.h>
 # include <gnutls/x509.h>
 # define HAVE_TLS
-# undef USE_OPENSSL
 #endif /* USE_GNUTLS and HAVE_GNUTLS */
 
-#if defined(USE_OPENSSL) && defined(HAVE_OPENSSL)
-# include <openssl/crypto.h>
-# include <openssl/x509.h>
-# include <openssl/pem.h>
-# include <openssl/ssl.h>
-# include <openssl/err.h>
-# include <openssl/rand.h>
-# include <openssl/md5.h>
-# define HAVE_SSL
-#endif /* USE_OPENSSL and HAVE_OPENSSL */
-
 #if defined(HAVE_TLS) || defined(HAVE_SSL)
 # define USE_SSL
 #else
@@ -151,9 +139,33 @@
 # include <libguile.h>
 #endif /* WITH_GUILE */
 
-#include "mem.h"		/* xfree(), xfree_pptr() */
+#include "xalloc.h"
+#include <argcv.h>
 #include "list.h"
 
+#ifdef HAVE_SYSEXITS_H
+# include <sysexits.h>
+#else
+# define EX_OK          0       /* successful termination */
+# define EX__BASE       64      /* base value for error messages */
+# define EX_USAGE       64      /* command line usage error */
+# define EX_DATAERR     65      /* data format error */
+# define EX_NOINPUT     66      /* cannot open input */
+# define EX_NOUSER      67      /* addressee unknown */
+# define EX_NOHOST      68      /* host name unknown */
+# define EX_UNAVAILABLE 69      /* service unavailable */
+# define EX_SOFTWARE    70      /* internal software error */
+# define EX_OSERR       71      /* system error (e.g., can't fork) */
+# define EX_OSFILE      72      /* critical OS file missing */
+# define EX_CANTCREAT   73      /* can't create (user) output file */
+# define EX_IOERR       74      /* input/output error */
+# define EX_TEMPFAIL    75      /* temp failure; user is invited to retry */
+# define EX_PROTOCOL    76      /* remote error in protocol */
+# define EX_NOPERM      77      /* permission denied */
+# define EX_CONFIG      78      /* configuration error */
+# define EX__MAX        78      /* maximum listed value */
+#endif
+
 #ifndef INADDR_NONE
 # define INADDR_NONE (unsigned long)0xffffffff
 #endif /* not INADDR_NONE */
@@ -214,7 +226,8 @@
 typedef enum anubis_mode
 {
   anubis_transparent,
-  anubis_authenticate
+  anubis_authenticate,
+  anubis_mda
 }
 ANUBIS_MODE;
 
@@ -274,10 +287,25 @@ typedef struct rc_regex RC_REGEX;
 typedef struct assoc ASSOC;
 typedef struct message_struct MESSAGE;
 
+#define xfree(p) do\
+	if (p) { \
+		free(p); \
+		p = NULL; \
+	}\
+     while (0)
+
+#define xfree_pptr(p) do\
+	if (p) { \
+		free_pptr(p); \
+		p = NULL; \
+	}\
+     while (0)
+
 /* stream.c */
 
 typedef struct net_stream *NET_STREAM;
-typedef int (*stream_io_t) (void *, char *, size_t, size_t *);
+typedef int (*stream_read_t) (void *, char *, size_t, size_t *);
+typedef int (*stream_write_t) (void *, const char *, size_t, size_t *);
 typedef int (*stream_close_t) (void *);
 typedef int (*stream_destroy_t) (void *);
 typedef const char *(*stream_strerror_t) (void *, int);
@@ -285,32 +313,23 @@ typedef const char *(*stream_strerror_t) (void *, int);
 void stream_create (NET_STREAM * str);
 int stream_set_io (NET_STREAM str,
 		   void *data,
-		   stream_io_t read, stream_io_t write,
+		   stream_read_t read, stream_write_t write,
 		   stream_close_t close,
 		   stream_destroy_t destroy, stream_strerror_t strerror);
-int stream_set_read (struct net_stream *str, stream_io_t read);
-int stream_set_write (struct net_stream *str, stream_io_t write);
+int stream_set_read (struct net_stream *str, stream_read_t read);
+int stream_set_write (struct net_stream *str, stream_write_t write);
 int stream_set_strerror (struct net_stream *str, stream_strerror_t strerr);
 int stream_close (NET_STREAM str);
 const char *stream_strerror (NET_STREAM str, int errcode);
 int stream_read (NET_STREAM str, char *buf, size_t size, size_t * nbytes);
-int stream_write (NET_STREAM str, char *buf, size_t size, size_t * nbytes);
+int stream_write (NET_STREAM str, const char *buf, size_t size,
+		  size_t * nbytes);
 int stream_readline (NET_STREAM str, char *buf, size_t size, size_t * nbytes);
 int stream_destroy (NET_STREAM *);
 
 /* main.c */
 void anubis (char *);
 
-/* mem.c */
-extern void (*memory_error) (const char *message);
-void *xmalloc (int);
-void *xrealloc (void *, int);
-char *allocbuf (char *, int);
-#ifndef HAVE_STRDUP
-char *strdup (const char *);
-#endif /* not HAVE_STRDUP */
-void free_pptr (char **);
-
 /* setenv.c */
 #if !defined(HAVE_SETENV) && defined(HAVE_PUTENV)
 int setenv (const char *, const char *, int);
@@ -319,8 +338,8 @@ int setenv (const char *, const char *, int);
 /* env.c */
 void get_options (int, char *[]);
 void get_homedir (char *, char *, int);
-void anubis_getlogin (char *, int);
-void anubis_changeowner (char *);
+void anubis_getlogin (char **);
+void anubis_changeowner (const char *);
 int anubis_set_mode (char *modename);
 int check_superuser (void);
 int check_username (char *);
@@ -330,8 +349,9 @@ void write_pid_file (void);
 
 /* errs.c */
 #define EXIT_ABORT 256
-void anubis_error (int, int, const char *, ...);
-void anubis_warning (int error_code, const char *fmt, ...);
+void anubis_error (int, int, const char *, ...) ANUBIS_PRINTFLIKE(3,4);
+void anubis_warning (int error_code, const char *fmt, ...)
+     ANUBIS_PRINTFLIKE(2,3);
 void socket_error (const char *);
 void hostname_error (char *);
 
@@ -344,9 +364,9 @@ void filelog (char *, char *);
 NET_STREAM make_remote_connection (char *, unsigned int);
 int bind_and_listen (char *, unsigned int);
 void swrite (int, NET_STREAM, char *);
-int recvline (int, NET_STREAM, void *, size_t);
-int recvline_ptr (int method, NET_STREAM sd, char **vptr, size_t * maxlen);
-void get_response_smtp (int, NET_STREAM, char *, int);
+void send_eol (int method, NET_STREAM sd);
+int recvline (int method, NET_STREAM sd, char **vptr, size_t * maxlen);
+void get_response_smtp (int, NET_STREAM, char **, size_t *);
 void close_socket (int sd);
 
 void net_create_stream (NET_STREAM * str, int fd);
@@ -358,18 +378,29 @@ void loop (int);
 void stdinout (void);
 void service_unavailable (NET_STREAM *);
 void set_unprivileged_user (void);
+void create_stdio_stream (NET_STREAM *s);
 
 /* auth.c */
-int auth_ident (struct sockaddr_in *, char *, int);
+int auth_ident (struct sockaddr_in *, char **);
 
 /* map.c */
-void parse_transmap (int *, char *, char *, char *, int);
+void parse_transmap (int *, char *, char *, char **);
 void translate_section_init (void);
 
 /* tunnel.c */
 void smtp_session (void);
 void smtp_session_transparent (void);
 void set_ehlo_domain (char *domain);
+void transfer_header (ANUBIS_LIST *);
+void transfer_body (MESSAGE *);
+void collect_headers (MESSAGE * msg, char *init_line);
+void collect_body (MESSAGE * msg);
+
+/* proclist.c */
+void proclist_register (pid_t pid);
+size_t proclist_cleanup (void (*fun) (size_t, pid_t, int));
+void proclist_init (void);
+size_t proclist_count (void);
 
 /* message.c */
 void message_add_body (MESSAGE *, char *, char *);
@@ -380,6 +411,7 @@ void message_modify_body (MESSAGE *, RC_REGEX *, char *);
 void message_external_proc (MESSAGE *, char **);
 void message_init (MESSAGE *);
 void message_free (MESSAGE *);
+void message_copy (MESSAGE *dst, MESSAGE *src);
 
 /* exec.c */
 char **gen_execargs (const char *);
@@ -391,15 +423,17 @@ void cleanup_children (void);
 /* esmtp.c */
 int esmtp_auth (NET_STREAM *, char *);
 void anubis_set_client_mech_list (ANUBIS_LIST *list);
+void anubis_set_encryption_mech_list (ANUBIS_LIST *list);
 
 /* misc.c */
 int anubis_free_list_item (void *item, void *data);
 void assoc_free (ASSOC *);
 ASSOC *header_assoc (char *);
+int anubis_assoc_cmp (void *item, void *data);
 void destroy_assoc_list (ANUBIS_LIST **);
 void destroy_string_list (ANUBIS_LIST **);
-void parse_mtaport (char *, char *, unsigned int *);
-void parse_mtahost (char *, char *, unsigned int *);
+void parse_mtaport (char *, char **, unsigned int *);
+void parse_mtahost (char *, char **, unsigned int *);
 void remline (char *, char *);
 void remcrlf (char *);
 char *substitute (char *, char **);
@@ -407,6 +441,8 @@ char *make_uppercase (char *);
 char *make_lowercase (char *);
 char *get_localname (void);
 char *get_localdomain (void);
+void assign_string (char **pstr, const char *s);
+void assign_string_n (char **pstr, const char *s, size_t length);
 
 /* mime.c */
 void message_append_text_file (MESSAGE *, char *);
@@ -431,8 +467,6 @@ void rcfile_call_section (int, char *, void *, MESSAGE *);
 char *user_rcfile_name (void);
 
 /* help.c */
-void print_version (void);
-void print_usage (void);
 void print_config_options (void);
 
 /* quit.c */
@@ -446,7 +480,7 @@ void quit (int);
 int check_socks_proxy (int, char *, unsigned int);
 #endif /* USE_SOCKS_PROXY */
 
-/* tls.c or ssl.c */
+/* tls.c */
 #ifdef USE_SSL
 void init_ssl_libs (void);
 NET_STREAM start_ssl_client (NET_STREAM str, const char *cafile, int verbose);
@@ -463,11 +497,7 @@ void gpg_section_init (void);
 /* guile.c */
 #ifdef WITH_GUILE
 void anubis_boot (void *, int, char **);
-void guile_load_path_append (char *);
 void guile_debug (int);
-void guile_load_program (char *);
-void guile_rewrite_line (char *, const char *);
-void guile_postprocess_proc (char *, ANUBIS_LIST **, char **);
 void guile_section_init (void);
 #endif /* WITH_GUILE */
 
@@ -554,12 +584,10 @@ void mysql_db_init (void);
 void pgsql_db_init (void);
 
 /* transmode.c */
-int anubis_transparent_mode (NET_STREAM * sd_client,
-			     struct sockaddr_in *addr);
+int anubis_transparent_mode (struct sockaddr_in *addr);
 
 /* authmode.c */
-int anubis_authenticate_mode (NET_STREAM * sd_client,
-			      struct sockaddr_in *addr);
+int anubis_authenticate_mode (struct sockaddr_in *addr);
 void anubis_set_password_db (char *arg);
 void asmtp_reply (int code, char *fmt, ...);
 void asmtp_capa_add_prefix (char *prefix, char *name);
@@ -568,8 +596,7 @@ void authmode_section_init (void);
 
 /* gsasl.c */
 void auth_gsasl_init (void);
-int anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr,
-		       NET_STREAM * stream);
+int anubis_auth_gsasl (char *auth_type, char *arg, ANUBIS_USER * usr);
 #ifdef WITH_GSASL
 void install_gsasl_stream (Gsasl_session_ctx * sess_ctx, NET_STREAM * stream);
 #endif
@@ -582,7 +609,7 @@ void anubis_set_server_mech_list (ANUBIS_LIST *list);
 
 /* xdatabase.c */
 int xdatabase (char *command);
-void xdatabase_capability (char *reply, size_t reply_size);
+void xdatabase_capability (char **preply, size_t *preply_size);
 void xdatabase_enable (void);
 
 /* md5.c */
@@ -592,4 +619,7 @@ int string_hex_to_bin (unsigned char *output, unsigned char *input, int inlen);
 
 #define MD5_DIGEST_BYTES 16
 
+/* mda.c */
+void mda (void);
+
 /* EOF */
