@@ -1,23 +1,19 @@
 /*
-   GNU Anubis v4.0 -- an SMTP message submission daemon.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   GNU Anubis v4.1 -- an SMTP message submission daemon.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -27,7 +23,7 @@ ANUBIS_MODE anubis_mode = anubis_transparent;
 
 const char version[] = "GNU Anubis v" VERSION;
 const char copyright[] =
-  "Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.";
+  "Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.";
 
 struct options_struct options;
 struct session_struct session;
@@ -39,7 +35,9 @@ unsigned long topt;
 NET_STREAM remote_client;
 NET_STREAM remote_server;
 
-char *anubis_domain;		/* Local domain for EHLO in authentication mode */
+char *anubis_domain;      /* Local domain for EHLO in authentication mode */
+char *incoming_mail_rule; /* Name of section for incoming mail processing */
+char *outgoing_mail_rule; /* Name of section for outgoing mail processing */
 
 #ifdef WITH_GUILE
 void
@@ -52,18 +50,15 @@ anubis_core (void)
 # define anubis_core() anubis(NULL)
 #endif /* WITH_GUILE */
 
-static void
-anubis_memory_error (const char *msg)
+void
+xalloc_die ()
 {
-  anubis_error (EXIT_FAILURE, 0, "%s", msg);
+  anubis_error (EXIT_FAILURE, 0, "%s", _("Not enough memory"));
 }
 
 int
 main (int argc, char *argv[])
 {
-  /* Register memory error printer */
-  memory_error = anubis_memory_error;
-
   /*
      Signal handling.
    */
@@ -102,8 +97,10 @@ main (int argc, char *argv[])
 
   SETVBUF (stderr, NULL, _IOLBF, 0);
   get_options (argc, argv);
-  anubis_getlogin (session.supervisor, sizeof (session.supervisor));
-
+  anubis_getlogin (&session.supervisor);
+  assign_string (&incoming_mail_rule, "INCOMING");
+  assign_string (&outgoing_mail_rule, "RULE");
+  
   /*
      Initialize various database formats
    */
@@ -158,7 +155,7 @@ main (int argc, char *argv[])
 	(int) geteuid (), (int) getegid ());
 
   /*
-     Initialize GnuTLS or OpenSSL library and the PRNG.
+     Initialize GnuTLS and the PRNG.
    */
 
 #ifdef USE_SSL
@@ -176,12 +173,12 @@ main (int argc, char *argv[])
 void
 anubis (char *arg)
 {
-  if (topt & T_STDINOUT)
-    {				/* stdin/stdout */
-      stdinout ();
-    }
+  if (anubis_mode == anubis_mda)  /* Mail Delivery Agent */
+    mda ();
+  else if (topt & T_STDINOUT)     /* stdin/stdout */
+    stdinout ();
   else
-    {				/* daemon */
+    {				  /* daemon */
       int sd_bind;
       sd_bind = bind_and_listen (session.anubis, session.anubis_port);
       if (topt & T_FOREGROUND_INIT)
