@@ -2,24 +2,20 @@
    misc.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -34,6 +30,12 @@ anubis_free_list_item (void *item, void *data)
   return 0;
 }
 
+int
+anubis_name_cmp (void *item, void *data)
+{
+  return strcmp (item, data);
+}
+
 /* String lists */
 
 void
@@ -102,27 +104,43 @@ assoc_to_header (ASSOC * asc)
   return buf;
 }
 
+int
+anubis_assoc_cmp (void *item, void *data)
+{
+  ASSOC *p = item;
+  return strcmp (p->key, data);
+}
+
+static void
+get_port_number (int *port, char *str)
+{
+  char *p;
+  *port = strtoul (str, &p, 0);
+  if (*p)
+    anubis_error (1, 0, _("Invalid port number: %s"), str);
+}
+
 /****************************
  An extra port number parser
 *****************************/
 
 void
-parse_mtaport (char *opt, char *host, unsigned int *port)
+parse_mtaport (char *opt, char **host, unsigned int *port)
 {
   struct servent *p;
-  char opt_tmp[256];
   char *port_tmp = NULL;
-  safe_strcpy (opt_tmp, opt);
 
-  if ((port_tmp = strrchr (opt_tmp, ':')))
+  if ((port_tmp = strrchr (opt, ':')))
     {
-      *port_tmp++ = '\0';
+      port_tmp++;
       if ((p = getservbyname (port_tmp, "tcp")))
 	*port = ntohs (p->s_port);
       else
-	*port = (unsigned int) atoi (port_tmp);
+        get_port_number (port, port_tmp);
+      assign_string_n (host, opt, port_tmp - opt - 1); 
     }
-  strncpy (host, opt_tmp, sizeof (session.mta) - 1);
+  else
+    assign_string (host, opt); 
 }
 
 /**************************
@@ -130,28 +148,26 @@ parse_mtaport (char *opt, char *host, unsigned int *port)
 ***************************/
 
 void
-parse_mtahost (char *opt, char *host, unsigned int *port)
+parse_mtahost (char *opt, char **host, unsigned int *port)
 {
   struct servent *p;
-  char opt_tmp[256];
-  char *port_tmp = NULL;
-  safe_strcpy (opt_tmp, opt);
+  char *port_tmp;
 
-  if ((port_tmp = strrchr (opt_tmp, ':')))
+  if ((port_tmp = strrchr (opt, ':')))
     {
-      *port_tmp++ = '\0';
+      port_tmp++;
       if ((p = getservbyname (port_tmp, "tcp")))
 	*port = ntohs (p->s_port);
       else
-	*port = (unsigned int) atoi (port_tmp);
-      strncpy (host, opt_tmp, sizeof (session.anubis) - 1);
+        get_port_number (port, port_tmp);
+      assign_string_n (host, opt, port_tmp - opt - 1);
     }
   else
     {				/* only port number available */
       if ((p = getservbyname (opt, "tcp")))
 	*port = ntohs (p->s_port);
       else
-	*port = (unsigned int) atoi (opt);
+        get_port_number (port, opt);
     }
 }
 
@@ -263,7 +279,7 @@ substitute (char *inbuf, char **subbuf)
   if (!inbuf || !subbuf)
     return NULL;
 
-  tmpbuf = allocbuf (inbuf, 0);
+  tmpbuf = xstrdup (inbuf);
   tmp++;
   while (*tmp)
     {
@@ -365,4 +381,28 @@ get_localdomain (void)
   return anubis_domain;
 }
 
+void
+assign_string (char **pstr, const char *s)
+{
+  free (*pstr);
+  if (s)
+    *pstr = xstrdup (s);
+  else
+    *pstr = NULL;
+}
+
+void
+assign_string_n (char **pstr, const char *s, size_t length)
+{
+  free (*pstr);
+  if (s)
+    {
+      *pstr = xmalloc (length + 1);
+      memcpy (*pstr, s, length);
+      (*pstr)[length] = 0;
+    }
+  else
+    *pstr = NULL;
+}
+
 /* EOF */
