@@ -2,24 +2,20 @@
    stream.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2004 The Anubis Team.
+   Copyright (C) 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -34,8 +30,8 @@ enum stream_state
 struct net_stream
 {
   enum stream_state state;
-  stream_io_t read;
-  stream_io_t write;
+  stream_read_t read;
+  stream_write_t write;
   stream_strerror_t strerror;
   stream_close_t close;
   stream_destroy_t destroy;
@@ -55,7 +51,7 @@ _def_strerror (void *data, int rc)
 }
 
 static int
-_def_write (void *sd, char *data, size_t size, size_t * nbytes)
+_def_write (void *sd, const char *data, size_t size, size_t * nbytes)
 {
   int rc = send ((int) sd, data, size, 0);
   if (rc >= 0)
@@ -88,13 +84,13 @@ _def_close (void *sd)
 void
 stream_create (struct net_stream **str)
 {
-  *str = xmalloc (sizeof **str);
+  *str = xzalloc (sizeof **str);
 }
 
 int
 stream_set_io (struct net_stream *str,
 	       void *data,
-	       stream_io_t read, stream_io_t write,
+	       stream_read_t read, stream_write_t write,
 	       stream_close_t close, stream_destroy_t destroy,
 	       stream_strerror_t strerror)
 {
@@ -111,7 +107,7 @@ stream_set_io (struct net_stream *str,
 }
 
 int
-stream_set_read (struct net_stream *str, stream_io_t read)
+stream_set_read (struct net_stream *str, stream_read_t read)
 {
   if (!str)
     return EINVAL;
@@ -120,7 +116,7 @@ stream_set_read (struct net_stream *str, stream_io_t read)
 }
 
 int
-stream_set_write (struct net_stream *str, stream_io_t write)
+stream_set_write (struct net_stream *str, stream_write_t write)
 {
   if (!str)
     return EINVAL;
@@ -176,7 +172,8 @@ stream_read (struct net_stream *str, char *buf, size_t size, size_t * nbytes)
 }
 
 int
-stream_write (struct net_stream *str, char *buf, size_t size, size_t * nbytes)
+stream_write (struct net_stream *str, const char *buf, size_t size,
+	      size_t *nbytes)
 {
   if (!str)
     return EINVAL;
