@@ -2,24 +2,20 @@
    tls.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /***********************
@@ -49,7 +45,7 @@ _tls_strerror (void *unused_data, int rc)
 }
 
 static int
-_tls_write (void *sd, char *data, size_t size, size_t * nbytes)
+_tls_write (void *sd, const char *data, size_t size, size_t * nbytes)
 {
   int rc;
 
@@ -376,7 +372,8 @@ cipher_info (gnutls_session session)
 	}
       print_x509_certificate_info (session);
       break;
-    case GNUTLS_CRD_SRP:
+
+    default:
       break;
     }
 
