@@ -2,24 +2,20 @@
    tunnel.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007 The Anubis Team.
 
-   GNU Anubis is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   GNU Anubis is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by the
+   Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
    GNU Anubis is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with GNU Anubis; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-   GNU Anubis is released under the GPL with the additional exemption that
-   compiling, linking, and/or using OpenSSL is allowed.
+   You should have received a copy of the GNU General Public License along
+   with GNU Anubis.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "headers.h"
@@ -31,10 +27,8 @@
 
 static int transfer_command (MESSAGE *, char *);
 static int process_command (MESSAGE *, char *);
-static void transfer_header (ANUBIS_LIST *);
-static void transfer_body (MESSAGE *);
 static void process_data (MESSAGE *);
-static int handle_ehlo (char *, char *, size_t);
+static int handle_ehlo (char *, char **);
 
 
 
@@ -106,13 +100,13 @@ add_header (ANUBIS_LIST * list, char *line)
     }
 }
 
-static void
-collect_headers (MESSAGE * msg)
+void
+collect_headers (MESSAGE * msg, char *line)
 {
-  char buf[LINEBUFFER + 1];
-  char *line = NULL;
-
-  while (recvline (SERVER, remote_client, buf, sizeof (buf) - 1))
+  char *buf = NULL;
+  size_t size = 0;
+  
+  while (recvline (SERVER, remote_client, &buf, &size))
     {
       remcrlf (buf);
       if (isspace ((u_char) buf[0]))
@@ -146,13 +140,13 @@ write_header_line (NET_STREAM sd_server, char *line)
 {
   char *p;
 
-  p = strtok (line, "\n");
+  p = strtok (line, "\r\n");
   do
     {
       swrite (CLIENT, sd_server, p);
-      swrite (CLIENT, sd_server, CRLF);
+      send_eol (CLIENT, sd_server);
     }
-  while ((p = strtok (NULL, "\n")));
+  while ((p = strtok (NULL, "\r\n")));
 }
 
 static void
@@ -207,11 +201,12 @@ send_string_list (NET_STREAM sd_server, ANUBIS_LIST * list)
 #define ST_BODY  2
 #define ST_DONE  3
 
-static void
+void
 collect_body (MESSAGE * msg)
 {
   int nread;
-  char buf[LINEBUFFER + 1];
+  char *buf = NULL;
+  size_t size = 0;
   struct obstack stk;
   int state = 0;
   int len;
@@ -223,7 +218,7 @@ collect_body (MESSAGE * msg)
     }
   obstack_init (&stk);
   while (state != ST_DONE
-	 && (nread = recvline (SERVER, remote_client, buf, sizeof (buf) - 1)))
+	 && (nread = recvline (SERVER, remote_client, &buf, &size)))
     {
       if (strncmp (buf, "." CRLF, 3) == 0)	/* EOM */
 	break;
@@ -262,6 +257,7 @@ collect_body (MESSAGE * msg)
 	  obstack_1grow (&stk, '\n');
 	}
     }
+  free (buf);
   obstack_1grow (&stk, 0);
   msg->body = strdup (obstack_finish (&stk));
   obstack_free (&stk, NULL);
@@ -275,9 +271,9 @@ send_body (MESSAGE * msg, NET_STREAM sd_server)
   if (msg->boundary)
     {
       swrite (CLIENT, sd_server, msg->boundary);
-      swrite (CLIENT, sd_server, CRLF);
+      send_eol (CLIENT, sd_server);
       send_string_list (sd_server, msg->mime_hdr);
-      swrite (CLIENT, sd_server, CRLF);
+      send_eol (CLIENT, sd_server);
     }
 
   for (p = msg->body; *p;)
@@ -289,14 +285,14 @@ send_body (MESSAGE * msg, NET_STREAM sd_server)
 	q = p + strlen (p);
 
       swrite (CLIENT, sd_server, p);
-      swrite (CLIENT, sd_server, CRLF);
+      send_eol (CLIENT, sd_server);
       p = q;
     }
 
   if (msg->boundary)
     {
       swrite (CLIENT, sd_server, msg->boundary);
-      swrite (CLIENT, sd_server, CRLF);
+      send_eol (CLIENT, sd_server);
     }
 }
 
@@ -308,7 +304,8 @@ send_body (MESSAGE * msg, NET_STREAM sd_server)
 void
 smtp_session_transparent (void)
 {
-  char command[LINEBUFFER + 1];
+  char *command = NULL;
+  size_t size = 0;
   MESSAGE msg;
 
   /*
@@ -316,9 +313,10 @@ smtp_session_transparent (void)
    */
 
   info (VERBOSE, _("Transferring message(s)..."));
-  get_response_smtp (CLIENT, remote_server, command, sizeof (command) - 1);
+  get_response_smtp (CLIENT, remote_server, &command, &size);
 
-  if (strncmp (command, "220 ", 4) == 0 && strstr (command, version) == 0)
+  if (command &&
+      strncmp (command, "220 ", 4) == 0 && strstr (command, version) == 0)
     {
       char *ptr = 0;
       char *banner_ptr = 0;
@@ -339,8 +337,9 @@ smtp_session_transparent (void)
 	    }
 	  while (*banner_ptr != ' ');
 	  banner_ptr++;
-	  snprintf (command, sizeof command,
-		    "220 %s (%s) %s", host, version, banner_ptr);
+	  command = xmalloc (4 + strlen (host) + 2 + strlen (version) + 2
+			     + strlen (banner_ptr) + 1);
+	  sprintf (command, "220 %s (%s) %s", host, version, banner_ptr);
 	}
     }
   swrite (SERVER, remote_client, command);
@@ -350,7 +349,7 @@ smtp_session_transparent (void)
    */
 
   message_init (&msg);
-  while (recvline (SERVER, remote_client, command, sizeof (command) - 1))
+  while (recvline (SERVER, remote_client, &command, &size))
     {
       if (process_command (&msg, command))
 	continue;
@@ -358,6 +357,7 @@ smtp_session_transparent (void)
       if (transfer_command (&msg, command) == 0)
 	break;
     }
+  free (command);
 
   message_free (&msg);
 }
@@ -365,21 +365,27 @@ smtp_session_transparent (void)
 void
 smtp_begin (void)
 {
-  char command[LINEBUFFER + 1];
-
+  char *command = NULL;
+  size_t size;
+  char *reply;
+  
   /* first get an mta banner */
-  get_response_smtp (CLIENT, remote_server, command, sizeof (command) - 1);
+  get_response_smtp (CLIENT, remote_server, &command, &size);
 
   /* now send the ehlo command */
-  snprintf (command, sizeof (command), "EHLO %s" CRLF, get_ehlo_domain ());
-  swrite (CLIENT, remote_server, command);
-  handle_ehlo (command, command, sizeof (command) - 1);
+  swrite (CLIENT, remote_server, "EHLO ");
+  swrite (CLIENT, remote_server, get_ehlo_domain ());
+  send_eol (CLIENT, remote_server);
+  handle_ehlo (command, &reply);
+  free (command);
+  free (reply);
 }
 
 void
 smtp_session (void)
 {
-  char command[LINEBUFFER + 1];
+  char *command = NULL;
+  size_t size = 0;
   MESSAGE msg;
 
   info (VERBOSE, _("Starting SMTP session..."));
@@ -387,7 +393,7 @@ smtp_session (void)
   info (VERBOSE, _("Transferring message(s)..."));
 
   message_init (&msg);
-  while (recvline (SERVER, remote_client, command, sizeof (command) - 1))
+  while (recvline (SERVER, remote_client, &command, &size))
     {
       if (process_command (&msg, command))
 	continue;
@@ -395,7 +401,7 @@ smtp_session (void)
       if (transfer_command (&msg, command) == 0)
 	break;
     }
-
+  free (command);
   message_free (&msg);
 }
 
@@ -470,19 +476,22 @@ handle_starttls (char *command)
   if (!(topt & T_LOCAL_MTA))
     {
       NET_STREAM stream;
-      char reply[LINEBUFFER + 1];
+      char *reply = NULL;
+      size_t size = 0;
       swrite (CLIENT, remote_server, "STARTTLS" CRLF);
 
-      get_response_smtp (CLIENT, remote_server, reply, sizeof (reply) - 1);
+      get_response_smtp (CLIENT, remote_server, &reply, &size);
 
       if (!isdigit ((unsigned char) reply[0])
 	  || (unsigned char) reply[0] > '3')
 	{
 	  remcrlf (reply);
 	  info (VERBOSE, _("WARNING: %s"), reply);
+	  free (reply);
 	  anubis_error (0, 0, _("STARTTLS command failed."));
 	  return 0;
 	}
+      free (reply);
 
       stream = start_ssl_client (remote_server,
 				 secure.cafile,
@@ -498,7 +507,7 @@ handle_starttls (char *command)
    */
 
   if (!secure.cert)
-    secure.cert = allocbuf (DEFAULT_SSL_PEM, MAXPATHLEN);
+    secure.cert = xstrdup (DEFAULT_SSL_PEM);
   if (!check_filename (secure.cert, NULL))
     {
       swrite (SERVER, remote_client,
@@ -507,7 +516,7 @@ handle_starttls (char *command)
     }
 
   if (!secure.key)
-    secure.key = allocbuf (secure.cert, MAXPATHLEN);
+    secure.key = xstrdup (secure.cert);
   else if (!check_filename (secure.key, NULL))
     {
       swrite (SERVER, remote_client,
@@ -584,26 +593,29 @@ save_ehlo_domain (char *command)
 }
 
 static int
-handle_ehlo (char *command, char *reply, size_t reply_size)
+handle_ehlo (char *command, char **reply)
 {
+  size_t reply_size = 0;
+  
+  *reply = NULL;
   if (!smtp_ehlo_domain_name)
     save_ehlo_domain (command);
     
-  get_response_smtp (CLIENT, remote_server, reply, reply_size - 1);
+  get_response_smtp (CLIENT, remote_server, reply, &reply_size);
 
-  if (strstr (reply, "STARTTLS"))
+  if (strstr (*reply, "STARTTLS"))
     topt |= T_STARTTLS;		/* Yes, we can use the TLS/SSL
 				   encryption. */
 
-  xdatabase_capability (reply, reply_size);
+  xdatabase_capability (reply, &reply_size);
 
 #ifdef USE_SSL
   if ((topt & T_SSL_ONEWAY)
       && (topt & T_STARTTLS) && !(topt & T_SSL_FINISHED))
     {
       NET_STREAM stream;
-      char ehlo[128];
-      char newreply[LINEBUFFER + 1];
+      char *newreply = NULL;
+      size_t nsize = 0;
 
       /*
          The 'ONEWAY' method is used when your MUA doesn't
@@ -616,17 +628,17 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
       info (NORMAL,
 	    _("Using TLS/SSL encryption between Anubis and remote MTA only..."));
       swrite (CLIENT, remote_server, "STARTTLS" CRLF);
-      get_response_smtp (CLIENT, remote_server, newreply,
-			 sizeof (newreply) - 1);
+      get_response_smtp (CLIENT, remote_server, &newreply, &nsize);
 
       if (!isdigit ((unsigned char) newreply[0])
 	  || (unsigned char) newreply[0] > '3')
 	{
 	  remcrlf (newreply);
 	  info (VERBOSE, _("WARNING: %s"), newreply);
+	  free (newreply);
 	  anubis_error (0, 0, _("STARTTLS (ONEWAY) command failed."));
 	  topt &= ~T_SSL_ONEWAY;
-	  swrite (SERVER, remote_client, reply);
+	  swrite (SERVER, remote_client, *reply);
 	  return 1;
 	}
 
@@ -636,7 +648,7 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
       if (!stream)
 	{
 	  topt &= ~T_SSL_ONEWAY;
-	  swrite (SERVER, remote_client, reply);
+	  swrite (SERVER, remote_client, *reply);
 	  return 1;
 	}
 
@@ -647,9 +659,10 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
          Send the EHLO command (after the TLS/SSL negotiation).
        */
 
-      snprintf (ehlo, sizeof (ehlo), "EHLO %s" CRLF, get_ehlo_domain ());
-      swrite (CLIENT, remote_server, ehlo);
-      get_response_smtp (CLIENT, remote_server, reply, reply_size);
+      swrite (CLIENT, remote_server, "EHLO ");
+      swrite (CLIENT, remote_server, get_ehlo_domain ());
+      send_eol (CLIENT, remote_server);
+      get_response_smtp (CLIENT, remote_server, reply, &reply_size);
     }
 #endif /* USE_SSL */
 
@@ -663,10 +676,10 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
     {
       char *starttls1 = "250-STARTTLS";
       char *starttls2 = "STARTTLS";
-      if (strstr (reply, starttls1))
-	remline (reply, starttls1);
-      else if (strstr (reply, starttls2))
-	remline (reply, starttls2);
+      if (strstr (*reply, starttls1))
+	remline (*reply, starttls1);
+      else if (strstr (*reply, starttls2))
+	remline (*reply, starttls2);
     }
 
   /*
@@ -675,7 +688,7 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
 
   if (topt & T_ESMTP_AUTH)
     {
-      char *p = strstr (reply, "AUTH ");
+      char *p = strstr (*reply, "AUTH ");
       if (p && (p[-1] == '-' || p[-1] == ' ') && memcmp (p-4, "250", 3) == 0)
 	{
 	  char *q = strchr (p, '\r');
@@ -693,27 +706,33 @@ handle_ehlo (char *command, char *reply, size_t reply_size)
 static int
 transfer_command (MESSAGE * msg, char *command)
 {
-  char reply[2 * LINEBUFFER + 1];
-  char buf[LINEBUFFER + 1];
-
-  safe_strcpy (buf, command);
+  char *reply = NULL;
+  size_t reply_size = 0;
+  char *buf = NULL;
+  int rc = 1; /* OK */
+  
+  assign_string (&buf, command);
   make_lowercase (buf);
+  
   swrite (CLIENT, remote_server, command);
 
   if (!strncmp (buf, "ehlo", 4))
     {
-      if (handle_ehlo (command, reply, sizeof (reply)))
-	return 0;
+      if (handle_ehlo (command, &reply))
+	{
+	  free (reply);
+	  return 0;
+	}
     }
   else
-    get_response_smtp (CLIENT, remote_server, reply, sizeof (reply));
+    get_response_smtp (CLIENT, remote_server, &reply, &reply_size);
 
   swrite (SERVER, remote_client, reply);
 
   if (isdigit ((unsigned char) reply[0]) && (unsigned char) reply[0] < '4')
     {
       if (strncmp (buf, "quit", 4) == 0)
-	return 0;		/* The QUIT command */
+	rc = 0;		/* The QUIT command */
       else if (strncmp (buf, "rset", 4) == 0)
 	{
 	  message_free (msg);
@@ -724,37 +743,41 @@ transfer_command (MESSAGE * msg, char *command)
 	  process_data (msg);
 	}
     }
-  return 1;			/* OK */
+  free (buf);
+  free (reply);
+  return rc;
 }
 
 void
 process_data (MESSAGE * msg)
 {
-  char buf[LINEBUFFER + 1];
+  char *buf = NULL;
+  size_t size = 0;
 
   alarm (1800);
 
-  collect_headers (msg);
+  collect_headers (msg, NULL);
   collect_body (msg);
 
-  rcfile_process_section (CF_CLIENT, "RULE", NULL, msg);
+  rcfile_call_section (CF_CLIENT, outgoing_mail_rule, NULL, msg);
 
   transfer_header (msg->header);
   transfer_body (msg);
 
-  recvline (CLIENT, remote_server, buf, sizeof (buf) - 1);
-  swrite (SERVER, remote_client, buf);
+  if (recvline (CLIENT, remote_server, &buf, &size))
+    swrite (SERVER, remote_client, buf);
+  free (buf);
 
   message_free (msg);
   message_init (msg);
   alarm (0);
 }
 
-static void
+void
 transfer_header (ANUBIS_LIST * header_buf)
 {
   send_header (remote_server, header_buf);
-  swrite (CLIENT, remote_server, CRLF);
+  send_eol (CLIENT, remote_server);
 }
 
 /***************
@@ -764,15 +787,15 @@ transfer_header (ANUBIS_LIST * header_buf)
 static void
 raw_transfer (void)
 {
-  int nread;
-  char buf[LINEBUFFER + 1];
-  while ((nread = recvline (SERVER, remote_client, buf,
-			    sizeof (buf) - 1)) > 0)
+  size_t size = 0;
+  char *buf = NULL;
+  while (recvline (SERVER, remote_client, &buf, &size) > 0)
     {
       if (strncmp (buf, "." CRLF, 3) == 0)	/* EOM */
 	break;
       swrite (CLIENT, remote_server, buf);
     }
+  free (buf);
 }
 
 void
@@ -787,7 +810,8 @@ transfer_body (MESSAGE * msg)
     }
   else
     send_body (msg, remote_server);
-  swrite (CLIENT, remote_server, "." CRLF);
+  if (anubis_mode != anubis_mda)
+    swrite (CLIENT, remote_server, "." CRLF);
 }
 
 /* EOF */
